Ext.define('DSS.component.CoolingPanel', {
	extend : 'Ext.panel.Panel',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    coolingModeSourceField:null,
    mapOrigin:null,
    coolingSystemStateSourceField:null,
    coolingSystemChangeOverSourceField:null,
    coolingPanel:null,
    coolingPanelIntro:null,
    basePanel:null,
    loadPanel: function () {
    	
    	var fHeatingMode = dss.ajaxSyncRequest('/json/property/query2', {
            'query' : '/usr/states/heating_water_system(callOrigin)'
        });
        result = Ext.JSON.decode(fHeatingMode);

        if (result.ok) {
            try {
                if (result.result['heating_water_system'].callOrigin != null) {
                    this.systemStatusOrigin = result.result['heating_water_system'].callOrigin;
                } else {
                    this.systemStatusOrigin = 0;
                }
                this.coolingModeSourceField.setValue(this.mapOrigin(this.systemStatusOrigin));
                if (this.systemStatusOrigin == CallOrigin.SystemBinaryInput) {
                    this.coolingSystemStateSourceField.enable();
                    this.coolingSystemChangeOverSourceField.enable();
                } else {
                    this.coolingSystemStateSourceField.disable();
                    this.coolingSystemChangeOverSourceField.disable();
                    this.coolingSystemStateSourceField.setValue(null);
                    this.coolingSystemChangeOverSourceField.setValue(null);
                }
            } catch (e) {
            }
        } else {
            success = false;
        }
        
        var fHeatingSystem = dss.ajaxSyncRequest('/json/property/query2', {
            'query' : '/usr/states/*(originDeviceDSUID)'
        });
        result = Ext.JSON.decode(fHeatingSystem);

        if (result.ok) {
            try {
                if (result.result['heating_system'].originDeviceDSUID != null && this.systemStatusOrigin == CallOrigin.SystemBinaryInput) {
                    this.coolingSystemStateSourceField.setValue(result.result['heating_system'].originDeviceDSUID);
                } else {
                    this.coolingSystemStateSourceField.setValue(null);
                }
                if (result.result['heating_system_mode'].originDeviceDSUID != null && this.systemStatusOrigin == CallOrigin.SystemBinaryInput) {
                    this.coolingSystemChangeOverSourceField.setValue(result.result['heating_system_mode'].originDeviceDSUID);
                } else {
                    this.coolingSystemChangeOverSourceField.setValue(null);
                }
            } catch (e) {
            }
        } else {
            success = false;
        }
    },
    loadTemperatureParameter : function(grid) {
		var success = this.basePanel.fetchZoneControlValues();

		var zoneID;
		var resultControlStatus, resultControlValues;
		var oResultObject = [];
        if (this.basePanel.temperatureControlValues) {
            for ( var sKey in this.basePanel.temperatureControlValues) {
                zoneID = sKey;
                resultControlStatus = this.basePanel.zoneControlValues[zoneID];
                resultControlValues = this.basePanel.temperatureControlValues[zoneID];
                zoneObject = {
                    zoneID : zoneID,
                    zoneName : dss.staticDataModel.getNameForZone(zoneID),
                    temperature : (resultControlStatus.TemperatureValue) ? resultControlStatus.TemperatureValue : null,
                    effectiveNominalTemperature : (resultControlStatus) ? resultControlStatus.NominalValue : null,
                    operationMode : (resultControlStatus) ? resultControlStatus.OperationMode : null,
                    nominalTemperatureComfort : ((resultControlValues) && (resultControlValues.Comfort > -43)) ? resultControlValues.Comfort : null,
                    nominalTemperatureEco : ((resultControlValues) && (resultControlValues.Economy > -43)) ? resultControlValues.Economy : null,
                    nominalTemperatureNotUsed : ((resultControlValues) && (resultControlValues.NotUsed > -43)) ? resultControlValues.NotUsed : null,
                    nominalTemperatureNight : ((resultControlValues) && (resultControlValues.Night > -43)) ? resultControlValues.Night : null,
                    nominalTemperatureHoliday : ((resultControlValues) && (resultControlValues.Holiday > -43)) ? resultControlValues.Holiday : null,
                    nominalTemperatureCooling : ((resultControlValues) && (resultControlValues.Cooling > -43)) ? resultControlValues.Cooling : null,
                    activeMode : resultControlValues.IsConfigured && ((resultControlStatus.ControlMode == 1) || (resultControlStatus.ControlMode == 3)),
                    inactiveMode : (resultControlValues) ? ((!resultControlValues.IsConfigured) || (resultControlStatus.ControlMode == 0) || (resultControlStatus.ControlMode == 4)
                            || (resultControlStatus.ControlMode == 2)) : null, // no
                    controlMode : (resultControlStatus) ? resultControlStatus.ControlMode : null,
                    roomSetback : resultControlStatus.roomSetback,
                    roomCoolingOff : resultControlStatus.roomCoolingOff
                };
                oResultObject.push(zoneObject);
            }
        }
		grid.loadParameter(oResultObject);
				
        return success;
	},
    constructor : function(config) {		
		var me=this;
		this.basePanel=config.basePanel;
        function formatCell(text, valid) {
            if (!valid) {
                text = '<span style="color:#aaaaaa;">' + text + '</span>';
            }
            return text;
        };
        var roomNameField = Ext.create('Ext.form.field.Display', {
            border : 0,
            height:18,
            fieldStyle: {'fontWeight' : 'bold'}
        });
        var coolingField = Ext.create('DSS.component.myTemperatureControl', {
            targetMode : 'nominalTemperatureCooling',
            label : _('Minimal temperature'),
            description : _('used in normal cooling operation')
        });

        var me = this;
        var oConfigCooling = {
            defaultSortCol : 'zoneID',
            inactiveField : 'inactiveMode',
            flex : 1,
            storeConfig : {
                storeId : 'temperatureStore',
                fields : [ {
                    name : 'zoneID'
                }, {
                    name : 'zoneName'
                }, {
                    name : 'nominalTemperatureCooling'
                }, {
                    name : 'controlMode'
                }, {
                    name : 'activeMode'
                }, {
                    name : 'inactiveMode'
                }, {
                    name : 'roomCoolingOff'
                }, {
                    name : 'disabled',
                    convert : function(value, record) {
                        return (record.get('roomCoolingOff') == true);
                    }
                } ]
            },
            columns : [ {
                header : _('zone'),
                dataIndex : 'zoneID',
                xtype : 'textcolumn',
                flex : 1,
                customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
                    var sText = dss.staticDataModel.getNameForZone(val);
                    return formatCell(sText, record.get('activeMode'));
                }
            }, {
                header : _('Minimal temperature'),
                dataIndex : 'nominalTemperatureCooling',
                flex : 1,
                xtype : 'textcolumn',
                customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
                    var sText = '-';
                    if (record.get('controlMode') !== ControlModes.cControlModeOff) {
                        sText = formatTemperature(val, record.get('controlMode'));
                    }
                    return formatCell(sText, record.get('activeMode'));
                }
            }, {
                header : _('Room cooling'),
                dataIndex : 'roomCoolingOff',
                resizable : false,
                xtype : 'disabledcolumn',
                width : 110
            } ],
            buttonText : {
                editButton : _('Edit settings')
            },
            editorConfig : {
                title : _('Cooling settings'),
                staticControls : [ {
                    type : 'hidden',
                    label : _('id'),
                    id : 'zoneID'
                }, {
                    type : 'hidden',
                    label : _('id'),
                    id : 'controlMode'
                }, {
                    type : 'custom',
                    control : roomNameField,
                    id : 'zoneName'
                }, {
                    type : 'custom',
                    control : coolingField,
                    id : 'nominalTemperatureCooling'
                } ]
            },
            listeners : {
                'loadDataStore' : function(grid) {
                    if (me.loadTemperatureParameter(grid)) {
                        grid.setStatus(_('Ready'));
                    } else {
                        grid.setStatus(_('Error'));
                    }
                },
                'saveEntry' : function(grid, values) {                                	
                	dss.pushAndQuery('heating-controller.operation-mode',{
            			actions : 'setTemperatureValues',
            			zoneID  : values.zoneID,
            			Cooling : values.nominalTemperatureCooling
            		},'heating-controller.answer',function(eventData) {
            			me.basePanel.fetchZoneControlValues(true);
            			me.loadTemperatureParameter(grid);
                        me.loadPanel();
            			return true;
            		});                  	                	
                },
                'enableEntry' : function(grid, entry) {
                    me.basePanel.setStartPanelStatus(_('changing Mode'));
                    var zoneID = entry.get('zoneID');

                	dss.pushAndQuery('heating-controller.operation-mode',{
                        actions : 'resetOperationMode',
                        zoneID : zoneID,
                        operationMode : OperationModes.Cooling
            		},'heating-controller.answer',function(eventData) {
            			me.basePanel.fetchZoneControlValues(true);
            			me.loadTemperatureParameter(grid);
                        me.loadPanel();
            			return true;
            		});
                },
                'disableEntry' : function(grid, entry) {
                    me.basePanel.setStartPanelStatus(_('changing Mode'));
                    var zoneID = entry.get('zoneID');
                	dss.pushAndQuery('heating-controller.operation-mode',{
                        actions : 'setOperationMode',
                        zoneID : zoneID,
                        operationMode : OperationModes.Cooling
            		},'heating-controller.answer',function(eventData) {
            			me.basePanel.fetchZoneControlValues(true);
            			me.loadTemperatureParameter(grid);
                        me.loadPanel();
            			return true;
            		});
                },
                'deleteEntry' : function(grid, entry) {
                },
                'testEntry' : function(grid, entry) {
                },
                'viewready' : function() {
                }
            }

        };
        
        this.mapOrigin = function(origin) {
            switch (origin) {
            case CallOrigin.SystemBinaryInput:
                return _('Internal sensor');
            case CallOrigin.DsmApi:
                return _('Building Management System');
            case CallOrigin.JSON:
                return _('Software interface');
            case CallOrigin.SystemStartup:
                return _('No sensor');
            }
            return _('Unknown');
        }
        this.coolingModeSourceField = Ext.create('Ext.form.field.Display', {
            labelWidth : 300,
            width : 700,
            fieldLabel: _('Heating/cooling mode selected by'),
            value: this.mapOrigin(this.systemStatusOrigin)
        });
        this.coolingSystemStateSourceField = Ext.create('Ext.form.field.Display', {
            labelWidth : 300,
            width : 700,
            fieldLabel: _('Heating on/off sensor'),
            renderer : function(val) {
                if (val != null && val != "") {
                    var oDevice = dss.staticDataModel.getDevice(val);
                    var sString = '<table><tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td>';
                    sString += '<td' + (oDevice.present ? '' : ' style="color:grey;"') + '>' + oDevice.name + '</td></tr></table>';
                    return sString;
                } else {
                    return _('-');
                }
            }
        });
        this.coolingSystemChangeOverSourceField = Ext.create('Ext.form.field.Display', {
            labelWidth : 300,
            width : 700,
            fieldLabel: _('Change-over signal sensor'),
            renderer : function(val) {
                if (val != null && val != "") {
                    var oDevice = dss.staticDataModel.getDevice(val);
                    var sString = '<table><tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td>';
                    sString += '<td' + (oDevice.present ? '' : ' style="color:grey;"') + '>' + oDevice.name + '</td></tr></table>';
                    return sString;
                } else {
                    return _('-');
                }
            }
        });
        this.coolingPanel = Ext.create('DSS.gridPanel', oConfigCooling);
        this.coolingPanelIntro = Ext.create('Ext.form.Panel', {
            border : 0,
            width : 800,
            height : 25,
            labelWidth: 800,
            padding : '5 0 0 5', // top right bottom left
            html : '<span style="color:#005138;">' + _("Note: these settings will be shown only once the cooling mode was set for the first time.") + '</span>'
        });

	    Ext.apply(this,  {
            border : false,
            flex : 1,
            style : {
                padding : '10px'
            },
            layout : {
                type : 'vbox',
                align : 'left'
            },
            items : [ {
                flex : 0,
                height : 150,
                width : "100%",
                xtype : 'panel',
                border : false,
                items : [ {
                    xtype : 'fieldset',
                    layout : 'vbox',
                    style : {
                        padding : '10px'
                    },
                    height : 144,
                    title : _('Cooling settings'),
                    items : [ this.coolingModeSourceField,
                              this.coolingSystemStateSourceField,
                              this.coolingSystemChangeOverSourceField,
                              this.coolingPanelIntro ]
                } ]
            }, {
                flex : 1,
                xtype : 'fieldset',
                style : {
                    padding : '10px'
                },
                layout : 'fit',
                reserveScrollbar : true,
                width : "100%",
                title : _('Roomsettings'),
                items : [ this.coolingPanel ]
            } ]
        });
	    this.loadPanel();
		this.callParent(arguments);
		this.loadTemperatureParameter(this.coolingPanel);
    }
});