Ext.define('DSS.component.HeatingPanel', {
	extend : 'Ext.panel.Panel',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    basePanel:null,
    preHeatingTimer :null,
    preHeatingHoliday:null,
    notUsedWhenAbsend:null,
    settingPersistentTemp:null,
    tempPanelIntro:null,
    temperaturePanel:null,
    constructor : function(config) {
    	this.basePanel=config.basePanel;
    	
    	
        this.preHeatingTimer = null;
		this.preHeatingHoliday = Ext.create('DSS.component.TimeSpanControl', {
			labelWidth : 295,
			width : 700,
			hideSecond : true,
			margin:'3 0 0 0',
            hourMaxValue : 72,
			fieldLabel : _('Preheating Time'),
			listeners : {
				change : function(oThis, newValue) {
                	dss.pushAndQuery('heating-controller.general-settings',{
						actions : 'setPreheatingTime',
						value : newValue
					},'heating-controller.answer',function(eventData) {
						me.savePreheatingTime();
						return true;
					});
				}
			}
		});
		
		this.notUsedWhenAbsend = Ext.create('Ext.form.Checkbox', {
			labelWidth : 295,
			width : 700,
			fieldLabel : _('Behavior on absent'),
			boxLabel : _('Should set mode Not Used when absent'),
			style : {
				padding : '10px',
				color : "#005138"
			},
			listeners : {
				change : function(oThis, newValue) {
					dss.pushAndQuery('heating-controller.general-settings',{
						actions : 'setUseNotUsedOnAbsent',
						value : newValue
					},'heating-controller.answer',function(eventData) {
						me.loadTemperatureParameter(me.temperaturePanel);
						return true;
					}); 
				}
			}
		});

		this.settingPersistentTemp  = Ext.create('Ext.form.RadioGroup', {
			labelWidth : 295,
			width : 700,
			fieldLabel : _('Keep temperature settings from room control unit'),
			items : [ {
				boxLabel : _('persistent'),
				name : 'settingPersistentTemp',
				inputValue : true,
				style : {
					padding : '10px',
					color : "#005138"
				}
			}, {
				boxLabel : _('until next heating mode change'),
				name : 'settingPersistentTemp',
				inputValue : false,
				style : {
					padding : '10px',
					color : "#005138"
				}
			}],
			listeners : {
				change : function(oThis, newValue, oldValue) {
					dss.pushAndQuery('heating-controller.general-settings',{
						actions : 'setPersistentNominalTemperature',
						value : newValue.settingPersistentTemp
					},'heating-controller.answer',function(eventData) {
						me.loadTemperatureParameter(me.temperaturePanel);
						return true;
					}); 
				}
			}
		});
    	
    	function formatCell(text, valid) {
			if (!valid) {
				text = '<span style="color:#aaaaaa;">' + text + '</span>';
			}
			return text;
		}
		;
		function createTooltip(record) {
			var sText = '';
			switch (record.get('effectiveNominalTemperature')) {
			case OperationModes.Comfort:
				if (record.get('nominalTemperatureComfort') == record.get('effectiveNominalTemperature')) {
					sText = _('current operation mode is comfort');
				} else {
					sText = _('current nominal temperature is set temporary');
				}
				break;
			case OperationModes.Eco:
				if (record.get('nominalTemperatureEco') == record.get('effectiveNominalTemperature')) {
					sText = _('current operation mode is economy');
				} else {
					sText = _('current nominal temperature is set temporary');
				}
				break;
			case OperationModes.NotUsed:
				if (record.get('nominalTemperatureNotUsed') == record.get('effectiveNominalTemperature')) {
					sText = _('current operation mode is not used');
				} else {
					sText = _('current nominal temperature is set temporary');
				}
				break;
			case OperationModes.Night:
				if (record.get('nominalTemperatureNight') == record.get('effectiveNominalTemperature')) {
					sText = _('current operation mode is night');
				} else {
					sText = _('current nominal temperature is set temporary');
				}
				break;
			case OperationModes.Holiday:
				if (record.get('nominalTemperatureHoliday') == record.get('effectiveNominalTemperature')) {
					sText = _('current operation mode is holiday');
				} else {
					sText = _('current nominal temperature is set temporary');
				}
				break;
			}
			return '<table><tr><td>' + sText + '</td></tr></table>';
		}
		;
        var roomNameField = Ext.create('Ext.form.field.Display', {
                    border : 0,
                    height:18,
                    fieldStyle: {'fontWeight' : 'bold'}                
        });
		var comfortField = Ext.create('DSS.component.myTemperatureControl', {
			targetMode : 'nominalTemperatureComfort',
			label : _('Comfort Mode'),
            description : _('used in normal operation when comfort mode is selected')
		});
		var ecoField = Ext.create('DSS.component.myTemperatureControl', {
			targetMode : 'nominalTemperatureEco',
			label : _('Eco Mode'),
            description : _('used in normal operation when eco mode is selected')
		});
		var notUsedField = Ext.create('DSS.component.myTemperatureControl', {
			targetMode : 'nominalTemperatureNotUsed',
			label : _('NotUsed Mode'),
            description : _('used if romm is not used')
		});
		var nightField = Ext.create('DSS.component.myTemperatureControl', {
			targetMode : 'nominalTemperatureNight',
			label : _('Night Mode'),
            description : _('used during planed night mode')
		});
		var holidayField = Ext.create('DSS.component.myTemperatureControl', {
			targetMode : 'nominalTemperatureHoliday',
			label : _('Holiday Mode'),
            description : _('used during defined holidays in Presence Simulation')
		});

		var me = this;
		var oConfigTemperature = {
			defaultSortCol : 'zoneID',
			inactiveField : 'inactiveMode',
            flex:1,
			storeConfig : {
				storeId : 'temperatureStore',
				fields : [ {
					name : 'zoneID'
				}, {
					name : 'zoneName'
                }, {
					name : 'temperature'
				}, {
					name : 'effectiveNominalTemperature'
				}, {
					name : 'nominalTemperatureComfort'
				}, {
					name : 'nominalTemperatureEco'
				}, {
					name : 'nominalTemperatureNotUsed'
				}, {
					name : 'nominalTemperatureNight'
				}, {
					name : 'nominalTemperatureHoliday'
				}, {
					name : 'activeMode'
				}, {
					name : 'inactiveMode'
				}, {
					name : 'controlMode'
				}, {
					name : 'operationMode'
				}, {
                    name : 'roomSetback'
                }, {
                    name : 'disabled',
                    convert : function(value, record) {
                        return record.get('roomSetback');
                    }
                } ]
			},
			columns : [ {
				header : _('zone'),
				dataIndex : 'zoneID',
				xtype : 'textcolumn',
				flex : 1,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sText = dss.staticDataModel.getNameForZone(val);
					return formatCell(sText, record.get('activeMode'));
				}
			}, {
				header : _('current Temperature'),
				dataIndex : 'temperature',
				xtype : 'textcolumn',
				flex : 1,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sText = formatTemperature(val, ControlModes.cControlModePID);
					return formatCell(sText, record.get('activeMode'));
				}
			}, {
				header : _('comfort mode temperature'),
				dataIndex : 'nominalTemperatureComfort',
				flex : 1,
				xtype : 'textcolumn',
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sText = '-';
					if (record.get('controlMode') !== ControlModes.cControlModeOff) {
						sText = formatTemperature(val, record.get('controlMode'));
					}
					return formatCell(sText, record.get('activeMode'));
				}

			}, {
				header : _('eco mode temperature'),
				dataIndex : 'nominalTemperatureEco',
				flex : 1,
				xtype : 'textcolumn',
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sText = '-';
					if (record.get('controlMode') !== ControlModes.cControlModeOff) {
						sText = formatTemperature(val, record.get('controlMode'));
					}
					return formatCell(sText, record.get('activeMode'));
				}

			}, {
				header : _('not used mode temperature'),
				dataIndex : 'nominalTemperatureNotUsed',
				flex : 1,
				xtype : 'textcolumn',
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sText = '-';
					if (record.get('controlMode') !== ControlModes.cControlModeOff) {
						sText = formatTemperature(val, record.get('controlMode'));
					}
					return formatCell(sText, record.get('activeMode'));
				}

			}, {
				header : _('night mode temperature'),
				dataIndex : 'nominalTemperatureNight',
				flex : 1,
				xtype : 'textcolumn',
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sText = '-';
					if (record.get('controlMode') !== ControlModes.cControlModeOff) {
						sText = formatTemperature(val, record.get('controlMode'));
					}

					return formatCell(sText, record.get('activeMode'));
				}

			}, {
				header : _('holiday mode temperature'),
				dataIndex : 'nominalTemperatureHoliday',
				flex : 1,
				xtype : 'textcolumn',
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sText = '-';
					if (record.get('controlMode') !== ControlModes.cControlModeOff) {
						sText = formatTemperature(val, record.get('controlMode'));
					}
					return formatCell(sText, record.get('activeMode'));
				}

			}, {
                header : _('RoomSetback Mode'),
                dataIndex : 'roomSetback',
                resizable : false,
                xtype : 'disabledcolumn',
                width : 110
            } ],
			buttonText : {
				editButton : _('Edit settings')
			},
			editorConfig : {
				title : _('Temperaturesettings'),
				staticControls : [ {
					type : 'hidden',
					label : _('id'),
					id : 'zoneID'
				}, {
					type : 'hidden',
					label : _('id'),
					id : 'controlMode'
				}, {
					type : 'custom',
                    control : roomNameField,
					id : 'zoneName'
				}, {
					type : 'custom',
					control : comfortField,
					id : 'nominalTemperatureComfort'
				}, {
					type : 'custom',
					control : ecoField,
					id : 'nominalTemperatureEco'
				}, {
					type : 'custom',
					control : notUsedField,
					id : 'nominalTemperatureNotUsed'
				}, {
					type : 'custom',
					control : nightField,
					id : 'nominalTemperatureNight'
				}, {
					type : 'custom',
					control : holidayField,
					id : 'nominalTemperatureHoliday'
				} ]
			},
			listeners : {
				'loadDataStore' : function(grid) {
                    if (me.loadTemperatureParameter(grid)) {
                        grid.setStatus(_('Ready'));
                    } else {
                        grid.setStatus(_('Error'));
                    }
				},
				'saveEntry' : function(grid, values) {
					me.saveTemperaturParameter(values);
				},
				'enableEntry' : function(grid, entry) {
                    me.basePanel.setStartPanelStatus(_('changing Mode'));
                    var zoneID = entry.get('zoneID');
                                                          
            		dss.pushAndQuery('heating-controller.operation-mode',{
                        actions : 'resetOperationMode',
                        zoneID : zoneID,
                        operationMode : OperationModes.NotUsed
            		},'heating-controller.answer',function(eventData) {
            			me.basePanel.fetchZoneControlValues(true);
            			me.reloadTemperatureSettings(me.temperaturePanel);
                    	me.basePanel.loadStartPanel();
            			return true;
            		});    			
				},
				'disableEntry' : function(grid, entry) {
                    me.basePanel.setStartPanelStatus(_('changing Mode'));
                    var zoneID = entry.get('zoneID');
                    
            		dss.pushAndQuery('heating-controller.operation-mode',{
                        actions : 'setOperationMode',
                        zoneID : zoneID,
                        operationMode : OperationModes.NotUsed
            		},'heating-controller.answer',function(eventData) {
            			me.basePanel.fetchZoneControlValues(true);
            			me.reloadTemperatureSettings(me.temperaturePanel);
                    	me.basePanel.loadStartPanel();
            			return true;
            		});    			
				}
			}

		};

		this.temperaturePanel = Ext.create('DSS.gridPanel', oConfigTemperature);
        this.tempPanelIntro = Ext.create('Ext.form.Panel', {
            border : 0,
            width : 800,
            height : 30,
            labelWidth: 800,
            padding : '0 0 10 5', // top right bottom left
            html : '<span style="color:#005138;">' + _("Define here the nominal temperature and control value for the different operation modes") + '</span>'
        });
 
	    Ext.apply(this, {
            border : false,
            flex : 1,
            style : {
                padding : '10px'
            },
            layout : {
                type : 'vbox',
                align : 'left'
            },
            items : [ {
                flex : 0,
                height : 120,
                width : "100%",
                xtype : 'panel',
                border : false,
                items : [ {
                    xtype : 'fieldset',
                    style : {
                        padding : '10px'
                    },
                    height : 114,
                    title : _('Heating settings'),
                    items : [
                        {
                            xtype : 'panel',
                            layout : 'hbox',
                            border : false,
                            margin : '0 0 6 0',
                            items : [  this.notUsedWhenAbsend ]
                        },
                        this.preHeatingHoliday,
                        this.settingPersistentTemp
                    ]
                } ]
            }, {
                flex : 1,
                xtype : 'fieldset',
                style : {
                    padding : '10px'
                },
                layout: {
                    type: 'vbox',
                    align : 'stretch',
                    pack : 'start'
                },
                reserveScrollbar : true,
                width : "100%",
                title : _('Roomsettings'),
                items : [ this.tempPanelIntro, this.temperaturePanel ]
            } ]
        });
		
		this.callParent(arguments);

    },
    savePreheatingTime : function() {
        this.preHeatingTimer = null;

        var preheatingTime = this.preHeatingHoliday.getValue();
		dss.raiseEvent('heating-controller.general-settings', {
			actions : 'setPreheatingTime',
			value : preheatingTime
		});
		this.loadTemperatureParameter(this.temperaturePanel);
	},
    loadTemperatureParameter : function(grid) {
		var success = this.basePanel.fetchZoneControlValues();

		var zoneID;
		var resultControlStatus, resultControlValues;
		var oResultObject = [];
        if (this.basePanel.temperatureControlValues) {
            for ( var sKey in this.basePanel.temperatureControlValues) {
                zoneID = sKey;
                resultControlStatus = this.basePanel.zoneControlValues[zoneID];
                resultControlValues = this.basePanel.temperatureControlValues[zoneID];
                zoneObject = {
                    zoneID : zoneID,
                    zoneName : dss.staticDataModel.getNameForZone(zoneID),
                    temperature : (resultControlStatus.TemperatureValue) ? resultControlStatus.TemperatureValue : null,
                    effectiveNominalTemperature : (resultControlStatus) ? resultControlStatus.NominalValue : null,
                    operationMode : (resultControlStatus) ? resultControlStatus.OperationMode : null,
                    nominalTemperatureComfort : ((resultControlValues) && (resultControlValues.Comfort > -43)) ? resultControlValues.Comfort : null,
                    nominalTemperatureEco : ((resultControlValues) && (resultControlValues.Economy > -43)) ? resultControlValues.Economy : null,
                    nominalTemperatureNotUsed : ((resultControlValues) && (resultControlValues.NotUsed > -43)) ? resultControlValues.NotUsed : null,
                    nominalTemperatureNight : ((resultControlValues) && (resultControlValues.Night > -43)) ? resultControlValues.Night : null,
                    nominalTemperatureHoliday : ((resultControlValues) && (resultControlValues.Holiday > -43)) ? resultControlValues.Holiday : null,
                    nominalTemperatureCooling : ((resultControlValues) && (resultControlValues.Cooling > -43)) ? resultControlValues.Cooling : null,
                    activeMode : resultControlValues.IsConfigured && ((resultControlStatus.ControlMode == 1) || (resultControlStatus.ControlMode == 3)),
                    inactiveMode : (resultControlValues) ? ((!resultControlValues.IsConfigured) || (resultControlStatus.ControlMode == 0) || (resultControlStatus.ControlMode == 4)
                            || (resultControlStatus.ControlMode == 2)) : null, // no
                    controlMode : (resultControlStatus) ? resultControlStatus.ControlMode : null,
                    roomSetback : resultControlStatus.roomSetback,
                    roomCoolingOff : resultControlStatus.roomCoolingOff
                };
                oResultObject.push(zoneObject);
            }
        }
		grid.loadParameter(oResultObject);
		
		
		this.preHeatingHoliday.suspendEvents();
		this.notUsedWhenAbsend.suspendEvents();
		this.settingPersistentTemp.suspendEvents();
		var sString = dss.ajaxSyncRequest('/json/property/query2', {
			'query' : '/scripts/heating-controller/generalSettings(*)/*(*)'
		});
		try {
			var result = Ext.JSON.decode(sString);
			if (result.ok) {
				if (result.result != null) {
					if (result.result.generalSettings != null) {
						if (result.result.generalSettings.useNotUsedOnAbsent != null) {
							this.notUsedWhenAbsend.setValue(result.result.generalSettings.useNotUsedOnAbsent);
						}
						if (result.result.generalSettings.preheatingTime != null) {
							this.preHeatingHoliday.setValue(result.result.generalSettings.preheatingTime);
						}
						if (result.result.generalSettings.persistNominalTemperature != null) {
							if (result.result.generalSettings.persistNominalTemperature) {
								this.settingPersistentTemp.setValue({settingPersistentTemp:true});
							} else {
								this.settingPersistentTemp.setValue({settingPersistentTemp:false});
							}
						}
					}
				}
			}

		} catch (e) {
		}
		this.preHeatingHoliday.resumeEvents();
		this.notUsedWhenAbsend.resumeEvents();
        this.settingPersistentTemp.resumeEvents();
		
        return success;
	},
	reloadTemperatureSettings : function(grid) {
		this.temperatureControlValues = null;
		this.loadTemperatureParameter(grid);
	},
	saveTemperaturParameter : function(values) {
		var me=this;
		dss.pushAndQuery('heating-controller.operation-mode',{
			actions : 'setTemperatureValues',
			zoneID  : values.zoneID,
			Comfort : values.nominalTemperatureComfort,
			Economy : values.nominalTemperatureEco,
			NotUsed : values.nominalTemperatureNotUsed,
			Night   : values.nominalTemperatureNight,
			Holiday : values.nominalTemperatureHoliday
		},'heating-controller.answer',function(eventData) {
			me.basePanel.fetchZoneControlValues(true);
			me.reloadTemperatureSettings(me.temperaturePanel);
        	me.basePanel.loadStartPanel();
			return true;
		});    			
	}
});
