function formatTemperature(val, controlMode) {
	if ((val === null) || (("" + val) == "")) {
		return '-';
	}

	var text;
	switch (controlMode) {
	case ControlModes.cControlModeZoneFollower:
		text = '-';
		break;
	case ControlModes.cControlModeFixedValue:
		text = ("" + val).replace('.', ',') + " %";
		break;
	default:
		val = Math.floor((val + 0.05) * 10) / 10;
		text = ("" + val).replace('.', ',') + " &deg;C";
		break;
	}
	return text;
}

function initializeConsole() {
	var method;
	var noop = function() {
	};
	var methods = [ 'info', 'log', 'time', 'timeEnd', 'debug', 'trace', 'warn' ];
	var length = methods.length;
	var console = (window.console = window.console || {});

	while (length--) {
		method = methods[length];

		// Only stub undefined methods.
		if (!console[method]) {
			console[method] = noop;
		}
	}
};

var HeatingControl;

function tabActivation(bsettingsMissing) {
	if (bsettingsMissing) {
		HeatingControl.tabControl.setActiveTab(HeatingControl.generalTab);
		HeatingControl.startTab.tab.setDisabled(false);
		HeatingControl.heatingTab.tab.setDisabled(true);
		HeatingControl.timeTab.tab.setDisabled(true);
		HeatingControl.coolingTab.tab.setDisabled(true);
	} else {
		HeatingControl.startTab.tab.setDisabled(false);
		HeatingControl.heatingTab.tab.setDisabled(false);
		HeatingControl.timeTab.tab.setDisabled(false);
		HeatingControl.coolingTab.tab.setDisabled(false);
	}
}

function getEmergencyValueForZone(zoneId) {
	var sString = dss.ajaxSyncRequest('/json/property/query2', {
		'query' : '/scripts/heating-controller/controllerConfig/' + zoneId + '(*)'
	});
	try {
		var result = Ext.JSON.decode(sString);
		var oObject = result.result[zoneId];
		if (oObject.emergenyValue != null) {
			var emergencyValue = parseInt("" + oObject.emergenyValue, 10);
			return emergencyValue;
		}
	} catch (e) {
	}
	return 75; // default value
}

Ext.define('DSS.addon.HeatingControl',
		{

			extend : 'DSS.addon.Framework',
			usableZones : [],

			possibleZones : [],   // imp
			tabControl : null,

			config : {
				appName : 'Heating Controller',
				appId : 'heating-controller',
				appVersion : '1.0.11',
				appIcon : 'default_icon.png'
			},
			getHelp : function() {
				return Ext.create('Ext.Component', {
                    whiteSpace : 'normal',
                    padding : '0 20 0 0', // top right bottom left
					loader : {
						url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
						autoLoad : true,
						disableCaching : false
					}
				});
			},
            startPanel : null,
            startPanelStatusText : null,
            startPanelStatusBar : null,
            tempPanelIntro : null,
			temperaturePanel : null,
			timePanel : null,
			generalPanel : null,
            heatingPanel : null,
			sensorPanel : null,
			generalSettingsControl : {},
			
			generalSettingsMissing : true,
			controllerDefined : false,
			zoneControlValues : null,
			overviewPanel:null,
            autoSelectMessage : [],   // imp
			heatingStore_controllerMode : null,
			heatingStore_referenceZone : null,
			systemStatus : SystemStatus.heating,
			hasCoolingCapability : false,
			systemStatusOrigin : 0,
			hasAVCapability:false,
			waitDialog : null,
			
			setStartPanelStatus: function (string) {
				this.startPanel.setStartPanelStatus(string);
			},
			loadStartPanel:function () {
				this.startPanel.loadPanel();
			},
			loadTimedSettings:function() {
				this.timePanel.loadTimedSettings();
				this.timePanel.updateTimeStore();
			},
			getContent : function() {
				var me = this;
				this.startPanel = Ext.create('DSS.component.StartPanel', {basePanel:this});

				this.fetchZoneControlValues();

                
				this.startTab = Ext.create('Ext.panel.Panel', {
					title : _('Overview'),
					layout : 'fit',
					items : [ this.startPanel ]
				});
				
				this.heatingPanel = Ext.create('DSS.component.HeatingPanel', {basePanel:this});
				this.heatingTab = Ext.create('Ext.panel.Panel', {
                    layout : 'fit',
					title : _('Heating'),
					items : [ this.heatingPanel ]
				});
				
				this.coolingTabPanel = Ext.create('DSS.component.CoolingPanel', {basePanel:this});
                this.coolingTab = Ext.create('Ext.panel.Panel', {
                    hidden : !this.hasCoolingCapability,
                    layout : 'fit',
                    title : _('Cooling'),
                    items : [ this.coolingTabPanel ]
                });
                
                
				this.timePanel = Ext.create('DSS.component.TimePanel', {basePanel:this});
				this.timeTab = Ext.create('Ext.panel.Panel', {
					layout : 'fit',
					title : _('Time profiles'),
					items : [ this.timePanel ]
				});
				
				this.ventilationTabPanel = Ext.create('DSS.component.VentilationPanel', {basePanel:this});
				this.ventilationTab = Ext.create('Ext.panel.Panel', {
					hidden : !this.hasAVCapability,
					layout : 'fit',
					title : _('Apartment Ventilation'),
					items : [ this.ventilationTabPanel ]
				});
				
				this.generalPanel = Ext.create('DSS.component.GeneralPanel', {basePanel:this});
				this.generalTab = Ext.create('Ext.panel.Panel', {
					layout : 'fit',
					title : _('General Settings'),
					items : [ this.generalPanel ]
				});
				
				this.tabControl = Ext.create('Ext.tab.Panel', {
					layout : 'fit',
					cls : 'generalTab',
					items : [ this.startTab, this.heatingTab, this.coolingTab,this.ventilationTab, this.timeTab, this.generalTab ]
				});

				return this.tabControl;
			},						
			fetchZoneControlValues : function(fReload) {
				var sString = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/scripts/heating-controller/generalSettings(*)/*(*)'
				});
				try {
					var result = Ext.JSON.decode(sString);
					if (result.ok) {
						if (result.result != null)
							if (result.result.generalSettings != null) {
		                        if ((result.result.generalSettings.capabilities != null)) {
		                            this.hasCoolingCapability = result.result.generalSettings.capabilities.coolingSupported;
		                            var hasHeatingCapability = result.result.generalSettings.capabilities.heatingSupported;
		                            if (this.coolingTab!=null) {
			                            if (this.hasCoolingCapability) {
			                                this.coolingTab.tab.show();
			                            } else {
			                                this.coolingTab.tab.hide();
			                            }
		                            }
		                        }
							}
					}

				} catch (e) {
				}

				var success = true;
                this.startPanel.setErrorState(false);
				if (fReload == null)
					if ((this.zoneControlValues != null) && (this.temperatureControlValues != null))
						return success;
				this.zoneControlValues = {};

				var sZoneControlStatus = dss.ajaxSyncRequest('/json/apartment/getTemperatureControlStatus', {});
				resultControlStatus = Ext.JSON.decode(sZoneControlStatus);
                if (!resultControlStatus.ok) {
                    console.log('failure in JSON request apartment/getTemperatureControlStatus');
                    success = false;
                    this.startPanel.setErrorState(true, 'Error getting current status of temperature control for all zones');
                }
				var oZones = dss.staticDataModel.getZoneArray();
				for ( var iIndex = 0; iIndex < oZones.length; iIndex++) {
					zoneID = oZones[iIndex].ZoneID;
					if (zoneID !== 0) { // all rooms
						for ( var jIndex = 0; jIndex < resultControlStatus.result.zones.length; jIndex++)
							if (resultControlStatus.result.zones[jIndex].id == zoneID)
								this.zoneControlValues[zoneID] = resultControlStatus.result.zones[jIndex];
					}
				}

				var sStringParameter = dss.ajaxSyncRequest('/json/apartment/getTemperatureControlConfig', {});
				var resultParameter = Ext.JSON.decode(sStringParameter);
        
				if (resultParameter.ok) {
					for (var iIndex=0;iIndex<resultParameter.result.zones.length;iIndex++) {
						this.zoneControlValues[resultParameter.result.zones[iIndex].id].config=resultParameter.result.zones[iIndex];
					}
                } else {
                    loadError = true;
                    this.startPanel.setErrorState(true, 'Error getting the temperature configuration control for all zones');
                }
				
				var sZoneSensorStatus = dss.ajaxSyncRequest('/json/apartment/getAssignedSensors', {});
				resultSensorStatus = Ext.JSON.decode(sZoneSensorStatus);
                if (!resultSensorStatus.ok) {
                    console.log('failure in JSON request apartment/getAssignedSensors');
                    success = false;
                    this.startPanel.setErrorState(true, 'Error getting list of assigned sensors for all zones');
                }

				for ( var iIndex = 0; iIndex < resultSensorStatus.result.zones.length; iIndex++) {
					var zoneID = resultSensorStatus.result.zones[iIndex].id;
					if (this.zoneControlValues[zoneID] != null)
						this.zoneControlValues[zoneID].sensors = resultSensorStatus.result.zones[iIndex].sensors;
				}
				this.usableZones = [];
				this.temperatureControlValues = {};
				
				var resultTemperature = Ext.JSON.decode(dss.ajaxSyncRequest('/json/apartment/getTemperatureControlValues', {}));
				
				for ( var sKey in this.zoneControlValues) {
					if ((this.zoneControlValues[sKey].ControlMode == 1) || (this.zoneControlValues[sKey].ControlMode == 3)) {
						for ( var iIndex = 0; iIndex < dss.staticDataModel.getZoneArray().length; iIndex++) {
							if (dss.staticDataModel.getZoneArray()[iIndex].ZoneID == parseInt(sKey, 10))
								this.usableZones.push(dss.staticDataModel.getZoneArray()[iIndex]);
						}

                        if (resultTemperature.ok) { 
                        	for (var iIndex=0;iIndex<resultTemperature.result.zones.length;iIndex++) {
                        		if (resultTemperature.result.zones[iIndex].id==sKey)
                        			this.temperatureControlValues[sKey] = resultTemperature.result.zones[iIndex];
                        	}                        	                            
                        } else {
                            console.log('failure in JSON request zone/getTemperatureControlValues');
                            success = false;
                            this.startPanel.setErrorState(true, 'Error getting the temperature control operation mode preset values for all zones');
                        }
					}
				}
				
				var resultSensorTemperature = Ext.JSON.decode(dss.ajaxSyncRequest('/json/apartment/getSensorValues', {}));				

				if (resultSensorTemperature.ok) {
					for (var iIndex=0;iIndex<resultSensorTemperature.result.zones.length;iIndex++) {
						for (var jIndex=0;jIndex<resultSensorTemperature.result.zones[iIndex].values.length;jIndex++) {
							if (resultSensorTemperature.result.zones[iIndex].values[jIndex].TemperatureValue!=null)
								this.zoneControlValues[resultSensorTemperature.result.zones[iIndex].id].actualTemperature=resultSensorTemperature.result.zones[iIndex].values[jIndex].TemperatureValue;							
						}
					}
				}
				
				if (Ext.data.StoreManager.lookup('usableZones') == null) {
					Ext.create('Ext.data.Store', {
						fields : [ 'id', 'name' ],
						storeId : 'usableZones',
						proxy : {
							type : 'memory',
							reader : {
								type : 'json',
								root : 'items'
							}
						}
					});
				}

				var newValue = {};
				for ( var iIndex = 0; iIndex < this.usableZones.length; iIndex++) {
					newValue['id'] = "" + this.usableZones[iIndex].ZoneID;
					newValue['name'] = "" + this.usableZones[iIndex].name;
					Ext.data.StoreManager.lookup('usableZones').add([ newValue ]);
				}

                this.possibleZones = [];
                var sString = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/apartment/dSMeters/*(dSUID)/*(temperature-control)/*(ZoneID)'
				});

                var result = Ext.JSON.decode(sString);
                for ( var sKey in result.result) {						
                    var oDSM = result.result[sKey];
                    if (oDSM.capabilities['temperature-control'] != null)
                        if (oDSM.capabilities['temperature-control']) {
                            for (aKey in oDSM.zones) {
                                var fFound = false;
                                aKey = parseInt('' + aKey, 10);
                                for ( var iIndex = 0; iIndex < this.possibleZones.length; iIndex++) {
                                    if ((""+this.possibleZones[iIndex]) == (""+aKey))
                                        fFound = true;
                                }
                                if (aKey!=0)
	                                if (!fFound) {                                	
	                                    this.possibleZones.push(aKey);										
	                                }
                            }
                        }
                }	
                
                /* now take apartment zone control values and add missing zones */
                for (var aKey in this.zoneControlValues) 
                    if (this.zoneControlValues[aKey].IsConfigured) {
                        var fFound=false;
                        for (var jIndex=0;jIndex<this.possibleZones.length;jIndex++){
                            if ((""+aKey)==(""+this.possibleZones[jIndex]))
                                fFound=true;
                        }
                        if (!fFound) {
                            this.possibleZones.push(this.zoneControlValues[aKey].id);
                        }
                    }
                
                var fApartmentEco = dss.ajaxSyncRequest('/json/property/query2', {
                    'query' : '/usr/addon-states/heating-controller/*(name,value)'
                });
                var result = Ext.JSON.decode(fApartmentEco);
                for ( var iIndex = 0; iIndex < this.possibleZones.length; iIndex++) {
                    var sZoneID = this.possibleZones[iIndex];
                    this.zoneControlValues[sZoneID].roomSetback = false;
                    this.zoneControlValues[sZoneID].roomCoolingOff = false;
                    if (this.zoneControlValues[sZoneID].IsConfigured) {
                        if (result.ok) {
                            if (result.result['operationmode.' + sZoneID + '.NotUsed'] != null) {
                                if (result.result['operationmode.' + sZoneID + '.NotUsed'].value != null) {
                                    this.zoneControlValues[sZoneID].roomSetback = (result.result['operationmode.' + sZoneID + '.NotUsed'].value == StateValue.inactive);
                                }
                            }
                            if (result.result['operationmode.' + sZoneID + '.Cooling'] != null) {
                                if (result.result['operationmode.' + sZoneID + '.Cooling'].value != null) {
                                    this.zoneControlValues[sZoneID].roomCoolingOff = (result.result['operationmode.' + sZoneID + '.Cooling'].value != StateValue.active);
                                }
                            }
                        }
                    }
                }

                if (this.heatingStore_referenceZone == null) {
					this.heatingStore_referenceZone = Ext.create('Ext.data.Store', {
						storeId : 'heatingStore_referenceZone',
						fields : [ 'id', 'name' ],
						proxy : {
							type : 'memory',
							reader : {
								type : 'json',
								root : 'items'
							}
						}
					});
				}
                this.heatingStore_referenceZone.removeAll();
				for ( var iIndex = 0; iIndex < this.usableZones.length; iIndex++) {
					var oObject = this.usableZones[iIndex];
					if (oObject.ZoneID != 0) {
						this.heatingStore_referenceZone.add({
							id : '' + oObject.ZoneID,
							name : oObject.name
						});
					}
				}
				var groups=dss.staticDataModel.getGroupForZone(0);
				for (var iIndex=0;iIndex<groups.length;iIndex++)
					if (groups[iIndex].group==64)
						this.hasAVCapability=groups[iIndex].reachable;

                return success;
			}
			
		});

Ext.onReady(function() {
	initializeConsole();
	dss.buildUpLang([ 'locale/{languageSuffix}/heating.po' ]);

	dss.raiseEvent('heating-controller.cloud', {
		actions: 'sync'
	});

	HeatingControl = Ext.create('DSS.addon.HeatingControl');
	HeatingControl.initPage();
	tabActivation(!HeatingControl.controllerDefined);
});
