Ext.define('DSS.component.StartPanel', {
	extend : 'Ext.panel.Panel',
    mixins : {
        observable : 'Ext.util.Observable'
    },
	loadError : {   errorPresent : false,
        errorDescr : ''
    }, 
    systemStatus:null,
    radioApartmentMode:null,
    targetTemperatureColumn:null,
    roomSetback:null,
    roomCooling:null,
    overviewPanel:null,
    sensorPanel:null,
    startPanelStatusText:null,
    startPanelStatusBar:null,
    loadPanel : function() {
    	
    },
    setErrorState : function(state, descr) {
        if ((state) && (state == true)) {
            this.loadError.errorPresent = state;
            this.loadError.errorDescr = descr;
        } else {
            this.loadError.errorPresent = false;
            this.loadError.errorDescr = '';
        }
    },

    setStartPanelStatus : function (text) {
        if (this.startPanelStatusText) {
            this.startPanelStatusText.setText(text);
        }
    },	
    coolingHeatingSwitch : function () {
        // change only if not in off mode
        if (this.basePanel.systemStatus != SystemStatus.off) {
            this.targetTemperatureColumn.setText((this.basePanel.systemStatus != SystemStatus.heating) ? _('Minimal temperature') : _('target Temperature'));
            this.roomSetback.setVisible(!(this.basePanel.systemStatus != SystemStatus.heating));
            this.roomCooling.setVisible((this.basePanel.systemStatus != SystemStatus.heating));
        }
    },
	loadPanel : function(fFirstLoad) {
        var success = true;
		// this.setStartPanelStatus(_('Refreshing...'))
		if (fFirstLoad == null)
			success = this.basePanel.fetchZoneControlValues(true);
		if (fFirstLoad == false)
			success = this.basePanel.fetchZoneControlValues(true);
		
		var fApartmentEco = dss.ajaxSyncRequest('/json/property/query2', {
			'query' : '/usr/addon-states/heating-controller/operationmode.0.Comfort(value)'
		});
		var result = Ext.JSON.decode(fApartmentEco);

		if (result.ok) {
			this.radioApartmentMode.suspendEvents();
			this.radioApartmentMode.setValue(null);
			try {
				if (result.result['operationmode.0.Comfort'].value == StateValue.inactive) {
					this.radioApartmentMode.setValue({
						'apartmentMode' : OperationModes.Eco
					});
				} else {
					this.radioApartmentMode.setValue({
						'apartmentMode' : OperationModes.Comfort
					});
				}
			} catch (e) {
			}
			this.radioApartmentMode.resumeEvents();
		} else {
            success = false;
        }
        
        var fHeatingControllerMode = dss.ajaxSyncRequest('/json/property/query2', {
            'query' : '/usr/addon-states/heating-controller/apartment_controller(state)'
        });
        result = Ext.JSON.decode(fHeatingControllerMode);

        if (result.ok) {
            this.systemStatus.setValue(_(SystemStatus.heating));
            try {
                this.basePanel.systemStatus = SystemStatus[result.result['apartment_controller'].state];
                this.systemStatus.setValue(_(this.basePanel.systemStatus));
                this.coolingHeatingSwitch();
            } catch (e) {
            }
        } else {
            success = false;
        }
                        

		if (fFirstLoad == null) {
			success = success && this.loadStatusParameter(this.overviewPanel);
        } else {
			if (fFirstLoad == false) {
				success = success && this.loadStatusParameter(this.overviewPanel);
            } else {
                this.updateSensorStatus([]);
            }
        }
		
		if (this.waitDialog != null) {
			this.waitDialog.hide();
			this.waitDialog = null;
		}
        if (success) {
            this.setStartPanelStatus(_('Ready'));
        } else {
            this.setStartPanelStatus(_('Error'));
        }
	},
	waitAndReloadStatus:function (grid) {
		var me = this;
		var iSubscriptID = Math.floor(Math.random() * 100000000000);
		dss.ajaxSyncRequest('/json/event/subscribe', {
			subscriptionID : iSubscriptID,
			name : 'heating-controller.answer'
		});
		dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
			subscriptionID : iSubscriptID,
			timeout : 60 * 1000
		}, function() {
			me.basePanel.zoneControlValues=null;
            if (me.loadStatusParameter(grid)) {
                me.setStartPanelStatus(_('Ready'));
            } else {
                me.setStartPanelStatus(_('Error'));
            }
			dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
				subscriptionID : iSubscriptID,
				name : 'heating-controller.answer'
			});
			
		});
	},
	loadStatusParameter : function(grid) {
		var success = this.basePanel.fetchZoneControlValues();
		var values = [];
        var messages=[];
		for ( var iIndex = 0; iIndex < this.basePanel.possibleZones.length; iIndex++) {
			var sZoneID = this.basePanel.possibleZones[iIndex];
			var oParameter = {
				zoneID : sZoneID,
				coolModeAvaible: true,
                emergencyMode: false,
                roomSetback: this.basePanel.zoneControlValues[sZoneID].roomSetback,
                roomCoolingOff: this.basePanel.zoneControlValues[sZoneID].roomCoolingOff
			};
			if (this.basePanel.zoneControlValues[sZoneID].IsConfigured) {
                if (this.basePanel.zoneControlValues[sZoneID].ControlState == 3) {
                    var sEmergencyValue = '(' +  this.basePanel.zoneControlValues[sZoneID].config.EmergencyValue + '%)';
                    //set error to messages
                    var msg = Ext.String.format2(_('Controller in {zonename} is in emergency state {emergencyvalue}'), {
                                    zonename : dss.staticDataModel.getNameForZone(sZoneID),
                                    emergencyvalue : sEmergencyValue});
                    messages.push(msg);
                    oParameter.emergencyMode = true;

                }
			}
			if (this.basePanel.zoneControlValues[sZoneID].actualTemperature != null)
				oParameter.actualTemperature = parseFloat(this.basePanel.zoneControlValues[sZoneID].actualTemperature);
			if (this.basePanel.zoneControlValues[sZoneID].NominalValue != null) {
				oParameter.type = this.basePanel.zoneControlValues[sZoneID].ControlMode;
				oParameter.actualTemperature = this.basePanel.zoneControlValues[sZoneID].TemperatureValue;
				oParameter.targetTemperature = this.basePanel.zoneControlValues[sZoneID].NominalValue;
			}
            if (this.basePanel.zoneControlValues[sZoneID].OperationMode!=null) {
                oParameter.actualMode = this.basePanel.zoneControlValues[sZoneID].OperationMode;
                //if control mode not 'off' and we are not in emergency mode set the cool mode enabler field visible
                if ((oParameter.actualMode != ControlModes.cControlModeOff) && (oParameter.emergencyMode === false)) {
                    oParameter.coolModeAvaible=false;
                }
            }
            // Override operation mode if systemStatus is off.
            if (this.systemStatus == SystemStatus.off) {
                oParameter.type = OperationModes.Off;
            }
            var oDevices = dss.staticDataModel.getDevicesForZone(sZoneID, 48);
            oParameter.actors = [];
            for ( var jIndex = 0; jIndex < oDevices.length; jIndex++) {
                if (oDevices[jIndex].outputMode > 0)
                    oParameter.actors.push(oDevices[jIndex]);
            }
            
            if ((this.basePanel.zoneControlValues[sZoneID].IsConfigured) && (oParameter.actors.length>0))
            	values.push(oParameter);
            else {
            	if (this.basePanel.zoneControlValues[sZoneID].actualTemperature!=null)
                	values.push(oParameter);
            }
		}
		
		grid.loadParameter(values);
        this.updateSensorStatus(messages);
		grid.setDisabled(false);
        return success;
	},
    updateSensorStatus : function (errMsgs) {
        var message=[];
		var sHTML='';
        var sGeneralErrorHTML='';
        //remote connectivity
        var fCloud=dss.isCloudAvaible(true);
		if (fCloud==false) {
			message.push(_('The remote connectivity is not configurated, so there will be some limitation when using the heating system via digitalSTROM'));
		}
        //general settings
        if (!this.controllerDefined) {
			message.push(_('Set the controller mode either manual or in the parameter of the device.'));                    
        }
		for (var iIndex=0;iIndex<message.length;iIndex++) {
			sGeneralErrorHTML+='<li style="color:#005138;list-style: disc outside none;">' + message[iIndex] + '</li>';
		}
        if (sGeneralErrorHTML != '') {
			sGeneralErrorHTML='<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">' + sGeneralErrorHTML + '</ul>';
        }
        //sensor error states
        var sSensorErrorHTML='';
        for (var iIndex=0;iIndex<errMsgs.length;iIndex++) {
			sSensorErrorHTML += '<li style="color:#005138;list-style: disc outside none;">' + errMsgs[iIndex] + '</li>';
		}
        if (sSensorErrorHTML != '') {
			sSensorErrorHTML='<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">' + sSensorErrorHTML + '</ul>';
        }
        //auto select errors
        var sAutoSelectHTML='';
        for (var jIndex = 0; jIndex < this.basePanel.autoSelectMessage.length; jIndex++) {
            sAutoSelectHTML += '<li style="color:#005138;list-style: disc outside none;">' +this.basePanel.autoSelectMessage[jIndex] + '</li>';
        }
        if (sAutoSelectHTML != '') {                     
            //sAutoSelectHTML += '<span style="color:#005138">' +  _('Please define the connected device for each actor under hardware / device properties or choose the operation mode manually') + '</span>';;
            sAutoSelectHTML = '<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">' + sAutoSelectHTML + '</ul>';
        }

        //load error
        message = [];
        if (this.loadError.errorPresent) {
            var sMessage = _('Error') + ': ' + this.loadError.errorDescr;
            message.push(sMessage);
        }
        var sLoadErrorHTML = '';
        if (message.length > 0) {
            sLoadErrorHTML = '<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">';
            for (var iIndex=0;iIndex<message.length;iIndex++) {
                sLoadErrorHTML+='<li style="color:#005138;list-style: disc outside none;">' + message[iIndex] + '</li>';
            }
            sLoadErrorHTML += '</ul>';
		}
        //now put it all together
        if (sGeneralErrorHTML != '') {
            if (sHTML === '') {
                sHTML += '<li style="color:#005138; list-style:none;">' + sGeneralErrorHTML + '</li>';
            } else {
                sHTML += '<li style="color:#005138; list-style:none;">&nbsp' + sGeneralErrorHTML + '</li>';                        
            }
        }
        if (sSensorErrorHTML != '') {
            if (sHTML === '') {
                sHTML += '<li style="color:#005138; list-style:none;">' + sSensorErrorHTML + '</li>';
            } else {
                sHTML += '<li style="color:#005138; list-style:none;">&nbsp' + sSensorErrorHTML + '</li>';
            }
        }
            
        if (sAutoSelectHTML != '') {
            if (sHTML === '') {
                sHTML += '<li style="color:#005138; list-style:none;">' + sAutoSelectHTML + '</li>';
            } else {
                sHTML += '<li style="color:#005138; list-style:none;">&nbsp' + sAutoSelectHTML + '</li>';
            }
        }
        if (sLoadErrorHTML != '') {
            if (sHTML === '') {
                sHTML += '<li style="color:#005138; list-style:none;">' + sLoadErrorHTML + '</li>';
            } else {
                sHTML += '<li style="color:#005138; list-style:none;">&nbsp' + sLoadErrorHTML + '</li>';
            }
        }
		if (sHTML=='') {
			sHTML = '<span style="color:#005138; list-style:none;">&nbsp' + _("No Messages") + '</span>';
        } else {
            sHTML = '<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">' + sHTML + '</ul>';
        }
        
		var item = Ext.create('Ext.panel.Panel', {
			border : false,
            overflowY: 'auto',
			html : sHTML
		});
        if (Ext.getCmp('sensorStatusPanel')) {
            Ext.getCmp('sensorStatusPanel').removeAll();
            Ext.getCmp('sensorStatusPanel').add(item);
        }
    },
	constructor : function(config) {
		var me = this;
		this.basePanel=config.basePanel;
		this.basePanel.startPanel=this;
		this.radioApartmentMode = Ext.create('Ext.form.RadioGroup', {
			labelWidth : 260,
			width : 700,
			fieldLabel : _('Standard heating mode'),
			items : [ {
				boxLabel : _('Comfort Mode'),
				name : 'apartmentMode',
				inputValue : OperationModes.Comfort,
				style : {
					padding : '10px',
					color : "#005138"
				}
			}, {
				boxLabel : _('Eco Mode'),
				name : 'apartmentMode',
				inputValue : OperationModes.Eco,
				style : {
					padding : '10px',
					color : "#005138"
				}
			} ],
			listeners : {
				change : function(oThis, newValue, oldValue) {

					me.waitDialog = new Ext.LoadMask(Ext.getBody(), {
						msg : _('Please wait, changing operation mode ...'),
						indicator : false
					});
					me.waitDialog.show();


					dss.raiseEvent('heating-controller.operation-mode', {
						actions : 'setOperationMode',
						zoneID : 0,
						operationMode : newValue.apartmentMode
					});

					var iSubscriptID = Math.floor(Math.random() * 100000000000);
					dss.ajaxSyncRequest('/json/event/subscribe', {
						subscriptionID : iSubscriptID,
						name : 'heating-controller.answer'
					});
					dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
						subscriptionID : iSubscriptID,
						timeout : 60 * 1000
					}, function() {
						dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
							subscriptionID : iSubscriptID,
							name : 'heating-controller.answer'
						});
						setTimeout(function() {me.loadPanel();}, 1000 );
					});

				}
			}
		});
		this.systemStatus = Ext.create('Ext.form.field.Display', {
            xtype: 'displayfield',
            labelWidth : 260,
            width : 700,
            fieldLabel : _('Heating controller status'),
            value: _('Heating')
		});
					
	    this.targetTemperatureColumn = Ext.create('DSS.component.textColumn', {
	        text : _('target Temperature'),
            dataIndex : 'targetTemperature',
            flex : 1,
            customColumnRenderer : function(val, metadata, record) {
                var sText = '-';
                if (record.get('type') !== ControlModes.cControlModeOff) {
                    sText = formatTemperature(val, record.get('type'));
                    if ((record.get('emergencyMode') === true) ||
                        (record.get('actualMode') === OperationModes.CoolingOff)) {
                        return '-';
                    }
                }
                return sText;
            }
	    });
	    
	    this.roomSetback = Ext.create('DSS.component.textColumn', {
            text : _('RoomSetback Mode'),
            dataIndex : 'roomSetback',
            width : 110,
            hidden : false,
            customColumnRenderer : function(val, metadata, record) {
                return record.get('roomSetback') ? _('off') : _('on');
            }
	    });
        this.roomCooling = Ext.create('DSS.component.textColumn', {
            text : _('Room cooling'),
            dataIndex : 'roomCoolingOff',
            width : 110,
            hidden : true,
            customColumnRenderer : function(val, metadata, record) {
                return record.get('roomCoolingOff') ? _('off') : _('on');
            }
        });

		var oConfigOverviewMode = {
			defaultSortCol : 'zoneID',
			storeConfig : {
				storeId : 'overviewStore',
				fields : [ {
					name : 'zoneID'
				}, {
					name : 'actualTemperature'
				}, {
					name : 'targetTemperature'
				}, {
					name : 'actualMode'
				}, {
					name : 'roomSetback'
				}, {
					name : 'type'							
				}, {
					name : 'coolModeAvaible'
				}, {
                    name : 'emergencyMode'
                }, {
                    name : 'roomCoolingOff'
                }]
			},
			disabledProperty:'disabled',
            showStatusBar : false,
            inactiveEnablerField: 'coolModeAvaible',
			columns : [ {
				header : _('zone'),
				dataIndex : 'zoneID',
				xtype : 'textcolumn',
				flex : 1,
				customColumnRenderer : function(val) {
					return dss.staticDataModel.getNameForZone(val);
				}
			}, {
				header : _('actual Temperature'),
				dataIndex : 'actualTemperature',
				xtype : 'textcolumn',
				flex : 1,
				customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
					var sText = formatTemperature(val, record.get('type'));
                    if (record.get('emergencyMode')===true) {
                        sText = '-'
                    }
					return sText;

				}
			},
			this.targetTemperatureColumn,
			{
				header : _('actual Mode'),
				dataIndex : 'actualMode',
				flex : 1,
				xtype : 'textcolumn',
				customColumnRenderer : function(val, metadata, record) {
                    if (record.get('emergencyMode')===true) {
                       return _('emergency state');
                    } else {
                        switch (val) {
                        case OperationModes.Off:
                            return _('Off Mode');
                            break;
                        case OperationModes.Comfort:
                            return _('Comfort Mode');
                            break;
                        case OperationModes.Eco:
                            return _('Eco Mode');
                            break;
                        case OperationModes.NotUsed:
                            return _('NotUsed Mode');
                            break;
                        case OperationModes.Night:
                            return _('Night Mode');
                            break;
                        case OperationModes.Holiday:
                            return _('Holiday Mode');
                            break;
                        case OperationModes.Cooling:
                            return _('Cooling Mode');
                            break;
                        case OperationModes.CoolingOff:
                            return _('CoolingOff Mode');
                            break;
                        }
                        return '-';
                    }
				}
			},
			this.roomSetback,
			this.roomCooling
            ],
			buttonText : {},
			editorConfig : {
				title : '',
				staticControls : [ {
					type : 'hidden',
					label : _('id'),
					id : 'zoneID'
				} ]
			},
			listeners : {
				'loadDataStore' : function(grid) {
					grid.setDisabled(true);
					me.setStartPanelStatus(_('changing Mode'));
					me.basePanel.zoneControlValues=null;
					if (me.loadStatusParameter(grid)) {
                        me.setStartPanelStatus(_('Ready'));
                    } else {
                        me.setStartPanelStatus(_('Error'));
                    }                            
				}
			}

		};

		this.overviewPanel = Ext.create('DSS.gridPanel', oConfigOverviewMode);

		this.sensorPanel = Ext.create('Ext.panel.Panel', {
			border : false
		});
        this.startPanelStatusText = Ext.create('Ext.toolbar.TextItem',{});
        this.startPanelStatusBar = Ext.create('Ext.toolbar.Toolbar', {
            width : '100%',
            items:[this.startPanelStatusText,
            '->',
                {
                    text:_('Refresh'),
                    handler: function(){
                        me.setStartPanelStatus(_('Refreshing...'));
                        me.loadPanel();
                    },
                    icon: dss.staticDataModel.baseImagePath + 'buttonIcons/refresh.png'
    			}]
        });

    	
	    Ext.apply(this,  {
			border : false,
			flex : 1,
			style : {
				padding : '10px'
			},
			layout : {
				type : 'vbox',
				align : 'left'
			},
			items : [ {
				height : 100,
				width : "100%",
				xtype : 'panel',
				border : false,
				items : [ {
					xtype : 'fieldset',
					style : {
						padding : '10px'
					},
					height : 94,
                    flex : 1,
					title : _('Apartment'),
					items : [ this.systemStatus, this.radioApartmentMode ]
				} ]
			}, {
				flex : 3,
				xtype : 'fieldset',
				style : {
					padding : '10px'
				},
				layout : 'fit',
				reserveScrollbar : true,
				width : "100%",
				title : _('Zones'),
				items : [ this.overviewPanel ]

			}, {
				flex : 1,
				xtype : 'panel',
				width : "100%",
				border : false,
				layout : 'fit',
				items : [ {
					xtype : 'fieldset',
					style : {
						padding : '10px'
					},
					flex : 1,
					layout : 'auto',
					height : "100%",
					title : _('Messages'),
					autoScroll : true,
					items : [ {
						xtype : 'panel',
						id : 'sensorStatusPanel',
						border : false
					} ]
				} ]
			}, 
            this.startPanelStatusBar 
        ]
		});
	    
		this.callParent(arguments);
		
		this.loadPanel();		
	}
});